#define MidiChannel 3
;#define ICD
;#define MIDITHROUGHTEST


; ppianopedal.asm
; Sept 2005
; by Johannes Taelman

		list p=18f2525		;list directive to define processor
		#include <p18f2525.inc>	;processor specific definitions

; #define simulate 1 ; enable simulation
; #define heartbeat 1 ; enable heatbeats
;******************************************************************************

#define	ErrorLed	LATC,	3;
#define	ClearErrorLed	BCF	ErrorLed
#define	SetErrorLed		BSF	ErrorLed

		#include "..\midi.asm"

;Configuration bits

	CONFIG	OSC = HSPLL
	CONFIG	FCMEN = ON
	CONFIG	IESO = ON
	CONFIG	PWRT = OFF
	CONFIG	BOREN = OFF	
	CONFIG	BORV = 1	
	CONFIG	WDT = OFF
	CONFIG	WDTPS = 1
	CONFIG	CCP2MX = PORTC
	CONFIG	STVREN = ON
	CONFIG	LVP = OFF

;----------------------------------------------------------------------------
; Variables
; memory map 
;----------------------------------------------------------------------------

		CBLOCK	0x000
		WREG_TEMP		;to save WREG during interrupt
		STATUS_TEMP		;to save STATUS during interrupt
		BSR_TEMP		;to save BSR during interrupt
		FSR0H_TEMP		;to save FSR0H during interrupt
		FSR0L_TEMP		;to save FSR0L during interrupt
		TBLPTRH_TEMP
		TBLPTRL_TEMP
		TABLAT_TEMP
		FSR0H_SHADOW		;to save FSR0H during high interrupt
		FSR0L_SHADOW		;to save FSR0L during high interrupt
		TmpM			; non-ISR scratch
		TmpT			; non-ISR scratch		
		TMidiInByte
		TMidiByte0
		TMidiByte1
		TMidiByte2
		ENDC
;----------------------------------------------------------------------------
; end of memory map
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;This code executes when a reset occurs.

		ORG     0x0000		;place code at reset vector

ResetVector:			bra	Main		;go to beginning of program

;----------------------------------------------------------------------------
;This code executes when a high priority interrupt occurs.

		ORG	0x0008

HighInt:
		btfsc	PIE2,TMR3IE	;else test if Timer3 interrupt enabled
		bra		Timer3ISR		;if so, go get received data
		reset			;error if no valid interrupt so reset

;----------------------------------------------------------------------------
;This code executes when a low priority interrupt occurs.

		ORG	0x0018

LowInt:	
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register
		movff	FSR0H,FSR0H_TEMP	;save FSR0H register
		movff	FSR0L,FSR0L_TEMP	;save FSR0L register
		movff	TBLPTRH,TBLPTRH_TEMP
		movff	TBLPTRL,TBLPTRL_TEMP
		movff	TABLAT, TABLAT_TEMP
		;test other interrupt flags here

;		btfss	PIR1,TMR1IF	;test for Timer1 interrupt flag
;		bra		LowInt1	
;		btfsc	PIE1,TMR1IE	;else test if Timer3 interrupt enabled
;		bra		Timer1ISR		;if so, go get received data
LowInt1:
		btfss	PIR1,RCIF	;test for RCIF receive interrupt flag
		bra		LowInt2	;if RCIF is not set, done with test
		btfsc	PIE1,RCIE	;else test if Rx interrupt enabled
		bra		GetData		;if so, go get received data
LowInt2:
		btfss	PIR1,TXIF	;test for TXIF transmit interrupt flag
		bra		LowInt3		;if TXIF is not set, done with test
		btfsc	PIE1,TXIE	;else test if Tx interrupt is enabled
		bra		PutData		;if so, go transmit data
LowInt3:
		reset			;error if no valid interrupt so reset

;		ORG     0x0300		;place code after midi code

		
;------------------------------------
;End of low priority interrupt routine restores context.

EndLowInt:	
		movff	TBLPTRH_TEMP,TBLPTRH
		movff	TBLPTRL_TEMP,TBLPTRL
		movff	TABLAT_TEMP,TABLAT
		movff	FSR0L_TEMP,FSR0L	;restore FSR0L register
		movff	FSR0H_TEMP,FSR0H	;restore FSR0H register
		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;----------------------------------------------------------------------------
;High priority interrupt routine.

HighIntCode:	
		btfss	PIR2,TMR3IF	;test for Timer3 receive interrupt flag
		bra		HighInt1	;if RCIF is not set, done with test
		btfsc	PIE2,TMR3IE	;else test if Timer3 interrupt enabled
		bra		Timer3ISR		;if so, go get received data
;can do special error handling here - an unexpected interrupt occurred 
HighInt1:	
		reset			;error if no valid interrupt so reset



;----------------------------------------------------------------------------
;Main routine checks for for reception of a <CR> and then calls a routine to
; move the data to transmit back.

Main:	
		setf	LATA	; Address strobes high
		clrf	LATB	; All outputs low
		setf	LATC	; All outputs low
	
		clrf 	TRISA 	; Config PORTA as all outputs
#ifdef ICD
		movlw	0xE0;
		movwf	TRISB
#else
		clrf 	TRISB 	; Config PORTB as all outputs
#endif
		movlw	0x80	;
		movwf 	TRISC 	; Config PORTC all outputs + tristate for serial rx
		movlw 	0x07	; init adc
		movwf	ADCON1	; init adc
		clrf 	ADCON0  ; poweroff adc
		clrf 	TRISA 	; Config PORTA again as all outputs

		call	LedGlowIn
;;		call	LedGlowIn
;;;		call	LedGlowOut ;defect
		
	
; init PWM
	movlw	0x7E;
	movwf	PR2
	clrf	CCPR1L
	clrf	CCPR2L
	movlw	0x0F;
	movwf	CCP1CON
	movwf	CCP2CON
	movlw 	0x05
	movwf 	T2CON
		movlw	0xFF;
		movwf	CCPR2L
		movlw	0xFF;
		movwf	CCPR1L


		lfsr  	FSR0 ,0000h
		lfsr  	FSR1 ,0000h
		rcall 	SetupMidi	;set up serial port and buffers


MainLoop:	
		call	PollMidiIn;
	bra	MainLoop	;go wait for more data


;----------------------------------------------------------------------------
; Timer3 setup

SetupTimer3:
		movlw 	0x01		
		movwf 	T3CON		; timer3 on, other bits cleared
		bsf		IPR2,1		; timer3 = high priority interrupt
		bsf		PIE2,1		; timer3 interrupt enable
	return		


Timer3ISR:
	retfie	FAST			;return and restore context
		

	ORG 0x500

MidiInCCAllNotesOff:
	RETURN

	return

MidiInCtrlChange:
		movlw	0x7B; 			CC NotesOff
		subwf	MidiByte1,0;
		bz		MidiInCCAllNotesOff;

		movlw	0x01; 			CC PWM
		subwf	MidiByte1,0;
		bz		MidiInCCPWM;

		movlw	0x02; 			CC PWM2
		subwf	MidiByte1,0;
		bz		MidiInCCPWM2;

		movlw	0x42; 			CC motor control on/off
		subwf	MidiByte1,0;
		bz		MidiInCCMotorOnOff;

		movlw	0x43; 			CC motor controller reset
		subwf	MidiByte1,0;
		bz		MidiInCCMotorReset;

	return


MidiInCCMotorOnOff:
		movf	MidiByte2,W
		addlw	0;
		bz		MidiInCCMotorOff;
		bcf		LATA,2;
	return

MidiInCCMotorOff:
		bsf		LATA,2;
	return	

MidiInCCMotorReset:
		bsf		LATA,3;
		movlw	0x08
		movwf	TMidiInByte
		clrf	TMidiByte0;
		clrf	TMidiByte1
_1outerloop1:	
	; inner loop
		movf	TMidiInByte,W;
		movwf	TMidiByte2
_1innerloop1:
_1ininnerloop1:
		MOVF	TMidiByte0,W
		CPFSGT	TMidiByte1
		SetErrorLed
		CPFSLT	TMidiByte1
		ClearErrorLed
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		incfsz	TMidiByte1
		bra 	_1ininnerloop1	
		decfsz	TMidiByte2
		bra 	_1innerloop1
		incfsz	TMidiByte0
		bra 	_1outerloop1	
		SetErrorLed
		clrf	TMidiByte0
		clrf	TMidiByte1
		clrf	TMidiByte2
		clrf	TMidiInByte

		bcf		LATA,3;		
	return

	
MidiInCCPWM:
		movf	MidiByte2,W
		xorlw	0x7F;
		movwf	CCPR1L
	return

MidiInCCPWM2:
		movf	MidiByte2,W
;		xorlw	0x7F;
		movwf	CCPR2L
	return


MidiInNoteOn:
		movf	MidiByte2,W;
		bz		MidiInNoteOffNoVelo;		; velocity=0 -> NOTE OFF
		
		movf	MidiByte1,W;
		xorlw	0x4C
		btfsc	STATUS,Z
		bsf		LATB,4
				
		movf	MidiByte1,W;
		xorlw	0x4D
		btfsc	STATUS,Z
		bsf		LATB,3

		movf	MidiByte1,W;
		xorlw	0x4E
		btfsc	STATUS,Z
		bsf		LATA,0
				
		movf	MidiByte1,W;
		xorlw	0x4F
		btfsc	STATUS,Z
		bsf		LATA,1

		movf	MidiByte1,W;
		xorlw	0x50
		btfsc	STATUS,Z
		bsf		LATA,2
				
		movf	MidiByte1,W;
		xorlw	0x51
		btfsc	STATUS,Z
		bsf		LATA,3

	return
	
MidiInNoteOffNoVelo:	
MidiInNoteOff:


		movf	MidiByte1,W;
		xorlw	0x4C
		btfsc	STATUS,Z
		bcf		LATB,4
				
		movf	MidiByte1,W;
		xorlw	0x4D
		btfsc	STATUS,Z
		bcf		LATB,3

		movf	MidiByte1,W;
		xorlw	0x4E
		btfsc	STATUS,Z
		bcf		LATA,0
				
		movf	MidiByte1,W;
		xorlw	0x4F
		btfsc	STATUS,Z
		bcf		LATA,1

		movf	MidiByte1,W;
		xorlw	0x50
		btfsc	STATUS,Z
		bcf		LATA,2
				
		movf	MidiByte1,W;
		xorlw	0x51
		btfsc	STATUS,Z
		bcf		LATA,3
		
	return


MidiInChannelPressure:
		movf	MidiByte1,W
;		xorlw	0x7F;
		movwf	CCPR2L		
	RETURN

MidiInProgramChange:
MidiInRTReset:
MidiInRTActiveSense:
MidiInRTStop:
MidiInRTContinue:
MidiInRTStart:
MidiInRTTick:
MidiInRTClock:
MidiInTuneRequest:
MidiInAfterTouch:
MidiInPitchWheel:
MidiInSysExStart:
MidiInSysExByte:
MidiInSysExEnd:
	RETURN


	END
